#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Exercice 4.1.4

def representation(n,B=2,k=None):
    """L’argument `n` doit être un entier naturel, et `B > 1`.
    Renvoie une liste `c` de coefficients telle que
    `n == sum( c[i]*(B**i) for i in range(len(c)) )`.
    Si `k` n’est pas fourni, la longueur de `c` est minimale.
    Si `k` est fourni, `len(c) == k` (une exception est soulevée
    si `k` est trop petit).
    """
    
    if B <= 1:
        raise ValueError("la base doit être > 1")
    if n < 0:
        raise ValueError("on représente les entiers naturels uniquement")
    c = []
    a = n 
    # On a : n == a*(B**len(c)) + valeur(c,B) et a >= 0
    while a > 0:
        # On maintient : n == a*(B**len(c)) + valeur(c,B) et a >= 0
        # De plus, a décroit strictement
        q, r = divmod(a, B)
        c.append(r)
        a = q
    if k is None:
        return c
    if k < len(c):
        raise ValueError("la longueur requise est insuffisante")
    return c+(k-len(c))*[0]

def valeur(c,B=2):
    """Renvoie `sum( c[i]*(B**i) for i in range(len(c)) )`, calculé à la main
    (ça évite de recalculer les puissances successives de `B`)."""
    
    n = 0
    puissance = 1 # Puissance de `B` courante
    for coefficient in c:
        n += coefficient * puissance
        puissance *= B
    return n
    
def conversion(c,B1,B2):
    return representation(valeur(c,B1),B2)

# Exercice 4.1.10

# Pour l'addition en taille fixe, on peut traduire directement la 
# définition par récurrence:
    
def addition_taille_fixe_rec(n,m,k,B=2):
    """Calcule la somme des écritures `n` et `m`
    de longueur `k` en base `B`."""
    
    if k == 0:
        return ([], 0)
    n_ = n[0:k-1]
    b = n[k-1]
    m_ = m[0:k-1]
    c = m[k-1]
    w, r = addition_taille_fixe_rec(n_, m_, k-1, B)
    s = b+c+r
    if s < B:
        return (w+[s], 0)
    else:
        return (w+[s-B],1)

# ou en donner une version itérative, un peu optimisée:
     
def addition_taille_fixe(n,m,k,B=2):
    """Calcule la somme des écritures `n` et `m`
    de longueur `k` en base `B`."""
    
    p = []
    r = 0
    for i in range(k):
        r, c = divmod(n[i] + m[i] + r, B)
        p.append(c)
    return (p,r)
# On propose deux solutions pour l’addition en taille variable:
#    * la première s’appuie sur l’addition en taille fixe
#    * la seconde reprend l’addition de l’école primaire

def addition(n,m,B=2):
    """Calcule la somme des écritures `n` et `m`
    en base `B`."""
    
    # on met d’abord les deux listes à la même longueur k
    k, lm = len(n), len(m)
    if k < lm:
        n = n + (lm-k)*[0]
        k = lm
    elif k > lm:
        m = m + (k-lm)*[0]
    # puis on utilise l'addition à taille fixe
    p, r  = addition_taille_fixe(n, m, k, B)
    if r:
        p.append(1)
    return p

def addition(n,m,B=2):
    """Calcule la somme des écritures `n` et `m`
    en base `B`."""
    
    i = 0
    p = []
    r = 0
    ln, lm = len(n), len(m)
    if lm < ln:
        # on s’assure que n est l’écriture la plus courte
        m, n, lm, ln = n, m, ln, lm
    for i in range(ln):
        r, q = divmod(n[i] + m[i] + r, B)
        p.append(q)
    for i in range(ln,lm):
        r, q = divmod(m[i] + r, B)
        p.append(q)        
    if r:
        p.append(1)
    return p
    
# Exercice 4.2.5

def representationZ(n,B=2,k=None):
    """L’argument `n` doit être un entier, et `B > 1`.
    Renvoie un couple `s,c` tel que `s == 0` ou `1`, et
    `n == s*B**k + sum( c[i]*(B**i) for i in range(len(c)) )`.
    Si `k` n’est pas fourni, la longueur de `c` est minimale.
    Si `k` est fourni, `len(c) == k` (une exception est soulevée
    si `k` est trop petit).
    """
    
    if B <= 1:
        raise ValueError("la base doit être > 1")
    if n >= 0:
        return representation(n,B,k),0
    
    l = 0
    puissance = 1
    while puissance < -n:
        puissance *= B
        l += 1
    
    if k is None:
        k = l
    if k < l:
        raise ValueError("la longueur requise est insuffisante")
    return representation(n+B**k,B,k),1

def valeur(c,s=0,B=2):
    """Renvoie la valeur de l’écriture signée `c,s` en base `B`.
    La seule différence avec la version pour les entiers naturels
    c’est qu’on soustrait B**len(c) au résultat si `s == 1`.
    """
    
    n = 0
    puissance = 1 # Puissance de `B` courante
    for coefficient in c:
        n += coefficient * puissance
        puissance *= B
    return n-s*puissance
    
def oppose(c,s=0,B=2):
    """Renvoie l’écriture de l’opposé de `valeur(c,s,B)`."""
    
    k = len(c)

    # Le cas `s == 0` est bien expliqué dans le cours 
    # mais il faut réfléchir un peu plus pour le cas `s == 1`.
    # Si `c` est une liste de `0`, ça représente
    # `-B**len(c)`, donc l’opposé nécessite un chiffre supplémentaire.    
    if s == 1 and all( c_i == 0 for c_i in c):
        # écriture de `B**len(c)`
        return c + [1], 0
    
    # Dans la suite, on sait que `-B**k < valeur(c,s,B) < B**k`.
    
    d = [B-c_i-1 for c_i in c] # complément à B-1
    un = [1]+(k-1)*[0] # écriture de 1 de même longueur que `c`
    
    e, t = addition_taille_fixe(d, un, k)
    # t == 1 seulement dans le cas où `d` est une liste de `B-1`
    # (cas où `c` est l’écriture de 0 uniquement vu qu’on a traité le cas de 
    # `-B**k` à part).
    
    # On a déjà les bons chiffres et il ne reste qu’à calculer le bit de signe :
    # * si `c` est une liste de zéros, on a `t == 1` et `s == 0`
    #   comme déjà expliqué, donc on veut renvoyer l’écriture de 0
    #   dont le bit de signe est 0 ;
    # * dans tous les autres cas, le bit de signe de l’opposé
    #   est obtenu en inversant `s`.
    return e, t^s^1

    
        
        
    