#!/usr/bin/env python3
# -*- coding: utf-8 -*-

def recherche_dichotomie(t, x):
    """Recherche l'indice de la première occurrence de `x` dans `t` par
    dichotomie, en supposant que `t` est trié (par valeurs croissantes).
    Renvoie `None` si `x` n'apparait pas dans `t`."""

    debut = 0
    fin = len(t)

    # On maintient l'invariant : `0 <= debut <= fin <= len(t)` et,
    # si `x` apparait dans `t`, alors l'indice `i` de sa première occurrence
    # vérifie `debut <= i < fin`.

    while fin > debut:
        longueur = fin - debut
        milieu = debut + longueur // 2
        # Comme `longueur > 0`, `0 <= longueur // 2 < longueur`,
        # donc `debut <= milieu < milieu+1 <= fin`

        # On considère une partition de `t` en trois morceaux :
        #  * de `debut` à `milieu-1` (vide si `début == milieu`)
        #  * la case d'indice `milieu`
        #  * de `milieu+1` à `fin-1` (vide si `milieu+1 == fin`)
        if debut < milieu and t[milieu-1] >= x:
            # On sait que s'il y a un `x` quelque part, le premier
            # se trouve entre `debut` et `milieu-1` (inclus) :
            # on change la `fin`.
            fin = milieu
        elif t[milieu] == x:
            # On sait que si `debut <= i < milieu`, alors `t[i] <= t[milieu-1] < x`
            # donc c'est bien la première occurrence et on termine.
            return milieu
        else:
            # On sait que si `debut <= i < milieu`, alors `t[i] <= t[milieu-1] < x`.
            # De plus `t[milieu] != x`. Donc s'il y a un `x` c'est après `milieu` :
            # on change le `debut`.
            debut = milieu+1

    # Si on arrive ici, c'est qu'on n'a pas trouvé `x`, vu que `fin==debut`.
    return None