#!/usr/bin/env python3
# -*- coding: utf-8 -*-

def recherche_dichotomie_bornes(t, x, debut, fin):
    """Recherche l'indice de la première occurrence de `x` dans `t`,
    située entre `debut` et `fin-1` (inclus), en supposant que `t` est trié
    (par valeurs croissantes), et `0 <= debut <= fin <= len(t)`.
    Renvoie `None` si `x` n'apparait pas dans cet intervalle."""

    if fin <= debut:
        # L'espace de recherche est vide
        return None

    longueur = fin - debut
    milieu = debut + longueur // 2
    # Comme `longueur > 0`, `0 <= longueur // 2 < longueur`,
    # donc `debut <= milieu < milieu+1 <= fin`

    # On considère une partition de l'espace de recherche en trois morceaux :
    #  * de `debut` à `milieu-1` (vide si `début == milieu`)
    #  * la case d'indice `milieu`
    #  * de `milieu+1` à `fin-1` (vide si `milieu+1 == fin`)
    if debut < milieu and t[milieu-1] >= x:
        # On sait que s'il y a un `x` dans l'intervalle, le premier
        # se trouve entre `debut` et `milieu-1` (inclus).
        return recherche_dichotomie_bornes(t, x, debut, milieu)
    elif t[milieu] == x:
        # On sait que si `debut <= i < milieu`, alors `t[i] <= t[milieu-1] < x`
        # donc c'est bien la première occurrence dans l'intervalle.
        return milieu
    else:
        # On sait que si `debut <= i < milieu`, alors `t[i] <= t[milieu-1] < x`.
        # De plus `t[milieu] != x`. Donc s'il y a un `x` dans l'intervalle
        # c'est après `milieu`.
        return recherche_dichotomie_bornes(t, x, milieu+1, fin)

def recherche_dichotomie(t, x):
    return recherche_dichotomie_bornes(t, x, 0, len(t))
