#!/usr/bin/env python3
# -*- coding: utf-8 -*-

## Itérations

def produit(t):
    p = 1
    for x in t:
        p = p*x
    return p

def indice_min(t):
    indice = 0
    valeur_min = t[0]
    for i in range(1,len(t)):
        if t[i] < valeur_min:
            indice = i
            valeur_min = t[i]
    return indice

def test_indice_min():
    for test in "indice_min([1,3,0,2])", "indice_min([1,2,1])":
        print(test, "→", eval(test))

## Variations sur le thème des itérations

def trouve_seuil(t,seuil):
    somme = 0
    taille = len(t)
    for i in range(taille):
        if somme >= seuil:
            return i
        somme += t[i]
    if somme >= seuil:
        return taille
    return None

def test_trouve_seuil():
    for i in range(3):
        print(i, "→", trouve_seuil((1,-1,1), i))

def produit_poly(p,q):
    if len(p) == 0 or len(q) == 0:
        return []
    r = []
    for degre in range(len(p)+len(q)-1):
        coefficient = 0
        # on calcule la somme des p[i]*q[j]
        # avec `i+j==degre`
        i_min = max(0,degre+1-len(q))
        i_max = min(len(p)-1,degre)
        for i in range(i_min,i_max+1):
            j = degre-i
            coefficient += p[i]*q[j]
        r.append(coefficient)
    return r

def test_produit_poly():
    tests = (
        ([0,0,1],[1,2,3]),
        ([],[1,2]),
        ([1,1],[1,1]),
        ([1,1],[-1,1]),
        )
    for p,q in tests:
        print(p,"×",q,"==",produit_poly(p,q))

## Première occurrence d’un élément

def indice(t,x):
    for i in range(len(t)):
        if t[i] == x:
            return i
    return None

def present(t,x):
    return indice(t,x) is not None
        
## Recherche d’élément satisfaisant un critere

def indice_critere(t,f):
    for i in range(len(t)):
        if f(t[i]):
            return i
    return None

def present_critere(t,f):
    return indice_critere(t, f) is not None

## Listes chaînées

vide = ()
def cons(x,u):
    return (x,u)
def te(u):
    x,_ = u
    return x
def qu(u):
    _,v = u
    return v

def longueur_liste(l):
    if l == ():
        return 0
    return 1+longueur_liste(qu(l))

def element_liste(l,i):
    if i == 0:
        return te(l)
    return element_liste(qu(l),i-1)

def tableau_vers_liste(t):
    """Construit une liste chainée contenant les éléments du tableau `t`."""
    l = vide
    # on construit la liste en partant de la fin, car `cons` ne peut
    # agir qu’en début de liste
    for i in range(len(t)):
        l = cons(t[-i-1],l)
    return l

def liste_vers_tableau(l):
    """Construit un tableau contenant les éléments de la liste `l`."""
    t = []
    while l != vide:
        t.append(te(l))
        l = qu(l)
    return t
    
## Remplacement de caractères

def remplace(d,mot):
    sortie = ""
    for x in mot:
        if x in d:
            sortie += d[x]
        else:
            sortie += x
    return sortie

def test_remplace():
    print(remplace({'c':'p', 'm':'r'},"acme"))
    
## Comptage

def compte(t):
    d = {}
    for x in t:
        if x in d:
            d[x] += 1
        else:
            d[x] = 1
    return d

## Syracuse

def syracuse(x):
    while not x <= 1:
        yield x
        if x % 2 == 0:
            x = x // 2
        else:
            x = 3*x+1
    yield x

def test_syracuse(x):
    """Affiche les termes, le temps de vol et l’altitude
    de la suite de Syracuse de premier terme x,
    cf. https://fr.wikipedia.org/wiki/Conjecture_de_Syracuse ."""
    
    suite = syracuse(x)
    temps = 0
    altitude = next(suite)
    print(f"Syracuse({x}) :")
    print(x*"*")
    for v in suite:
        print(v*"*")
        altitude = max(v, altitude)
        temps += 1
    print()
    print("Altitude:",altitude,"Temps:",temps)
        
    
    
