#!/usr/bin/env python3
# -*- coding: utf-8 -*-

class GrapheOriente:
    
    """Une classe pour les graphes orientés.
    
    L'attribut `sommets` est un dictionnaire dont les clés sont les sommets.
    Pour chaque sommet `x`, `sommets[x]` est un couple `(entrants,sortants)`
    où `entrants` est l’ensembles des arcs entrants de `x`, et 
    `sortants` l’ensemble des arcs sortants.
    
    L'attribut `arcs` est un dictionnaire dont les clés sont les arcs.
    Pour chaque arc `a`, `arcs[a]` est un couple `(source,but)`, où
    `source` et `but` sont les sommets source et but de `a`.
    """

    def __init__(self, S=set(), A={}):
        
        """Crée un nouveau graphe à partir d'un ensemble de sommets `S` 
        (vide par défaut), et d'un ensemble d'arcs `A` (vide par défaut)
        chacun muni d'un sommet source et d'un sommet but.
        
        Formellement : `S` est un itérable et `A` est un tableau clé-valeur:
        les clés de `A` sont les arcs, et la valeur associée à un arc est 
        le couple formé de son sommet source et son sommet but."""
        
        self.sommets = {}
        self.arcs = {}
        for x in S:
            self.ajoute_sommet(x)
        for a in A:
            s,t = A[a]
            self.ajoute_arc(a, s, t)

    def ajoute_sommet(self, x):
        if x in self.sommets:
            raise ValueError("le sommet existe déjà")
        self.sommets[x] = (set(), set())

    def ajoute_arc(self, a, s, t):
        if a in self.arcs:
            raise ValueError("l'arc existe déjà")
        self.arcs[a] = (s,t)
        self.sommets[s][1].add(a) # ajoute `a` aux arcs sortants de `s`
        self.sommets[t][0].add(a) # ajoute `a` aux arcs entrants de `t`

    def source(self, a):
        return self.arcs[a][0]

    def but(self, a):
        return self.arcs[a][1]

    def entrants(self, x):
        return self.sommets[x][0]

    def sortants(self, x):
        return self.sommets[x][1]
